# -*- coding: utf-8 -*-
'''
this is a part of the heybuddy project
copyright 2010 jezra lickter http://www.jezra.net

copyright 2012 Daniel Martí; thanks buddy!
'''
import httplib
import re
import socket
import urllib2
try:
	import ssl
	has_ssl_module = True
except:
	 has_ssl_module = False
	 print '''-- SSL module was not detected --
It is **HIGHLY** recommended that you install the SSL Module available at:
http://pypi.python.org/pypi/ssl/
	
Without this module, Heybuddy can not verify the credentials of a status.net instance.	
	
	'''

class InvalidCertificateException(httplib.HTTPException, urllib2.URLError):
	def __init__(self, host, certificate, reason):
		httplib.HTTPException.__init__(self)
		self.host, self.certificate, self.reason = host, certificate, reason
	def __str__(self):
		return "Host "+self.host+" proportioned an invalid certificate: "+self.reason+" - "+self.certificate
		  
class CertValidatingHTTPSConnection(httplib.HTTPConnection):
	default_port = httplib.HTTPS_PORT
	def __init__(self, host, port=None, key_file=None, cert_file=None, ca_certs=None, strict=None, **kwargs):
		httplib.HTTPConnection.__init__(self,host,port,strict,**kwargs)
		self.key_file = key_file
		self.cert_file = cert_file
		self.ca_certs = ca_certs
		if self.ca_certs is None:
			cert_reqs = ssl.CERT_NONE
		else:
			cert_reqs = ssl.CERT_REQUIRED
		self.cert_reqs = cert_reqs
			
	def get_valid_hosts_for_certificate(self, certificate):
		hosts = []
		if 'subjectAltName' in certificate:
			for element in certificate['subjectAltName']:
				if element[0].lower() == 'dns':
					hosts.append( element[1] )
		else:
			for element in certificate['subject']:
				if element[0][0].lower() == 'commonname':
					hosts.append(element[0][1])
		return hosts

	def validate_certificate_hostname(self, certificate, hostname):
		hosts = self.get_valid_hosts_for_certificate(certificate)
		for host in hosts:
			regex_host = '^'+host.replace('.', '\.').replace('*', '[^.]*')+'$'
			if re.search(regex_host, hostname, re.I):
				return True
		return False

	def connect(self):
		conn = socket.create_connection((self.host, self.port))
		self.sock = ssl.wrap_socket(conn, keyfile=self.key_file, certfile=self.cert_file, cert_reqs=self.cert_reqs, ca_certs=self.ca_certs)
		if self.cert_reqs and ssl.CERT_REQUIRED:
			cert = self.sock.getpeercert()
			hostname = self.host.split(':', 0)[0]
			# -- if self.validate_certificate_hostname( cert, hostname) 
			# Don't you mean 'if not'?
			if not self.validate_certificate_hostname(cert, hostname):
				raise InvalidCertificateException(hostname, cert, 'hostname mismatch')
				
class VerifiedHTTPSHandler(urllib2.HTTPSHandler):
	def __init__(self, **kwargs):
		 urllib2.AbstractHTTPHandler.__init__(self)
		 self.connection_args = kwargs

	def https_open(self, request):
		def http_class_wrapper(host,**kwargs):
			all_kwargs = dict(self.connection_args)
			all_kwargs.update(kwargs)
			return CertValidatingHTTPSConnection(host, **all_kwargs)
		try:
			return self.do_open(http_class_wrapper,request)
		except urllib2.URLError, e:
			if type(e.reason) == ssl.SSLError and e.reason.args [0] == 1:
				 raise InvalidCertificateException(request.host, '', e.reason.args[1])
			raise

	https_request = urllib2.HTTPSHandler.do_request_
